#ifndef CMDPARSER_HPP
#define CMDPARSER_HPP 1

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/path.hpp"

#include <iostream>
#include <popt.h>
#include <sstream>
#include <string>

/* #define CMDDEBUG */

/* This class holds the complete configuration for mbox2eml. */
class Configuration {
    public:
        /* the input mbox */
        boost::filesystem::path input_file;
        /* the output directory */
        boost::filesystem::path output_dir;

        /* shall unbitstuffing be performed? */
        bool unbitstuff;
        /* This is the character, which will be removed when 'unbitstuff'ing. */
        char unbitstuff_char;
        /* These are the characters which may be between the 'unbitstuff-char'
         * and 'From'. If only these characters are found, the affected line is
         * 'unbitstuff'ed.
         */
        std::string unbitstuff_targets;
        /* trailing space after 'From' necessary? */
        bool trailing_space;

        Configuration();
};

class Cmdline_parser {
    private:
        /* This function checks the submitted directory name for:
         * - correctness of the name
         * - existance
         * If the directory does not exist and the name is valid,
         * a new dirctory is created.
         *
         * If the directory name exists but is no directory, an error is raised.
         *
         * Returns: the verified path of the directory
         */
        boost::filesystem::path check_dir(const char* dir_str);

        /* This function checks the submitted file name for existance.
         * If the file name exists but is no file, an error is raised.
         *
         * Returns: the verified path of the file
         */
        boost::filesystem::path check_file(const char* file_str);

        /* Here, all obligatory options are checked for existance. */
        bool check_necessary_options_set(Configuration* config);

        /* returns the usage instructions
         *
         * if oneliner==true only one line is returned
         */
        std::string usage(bool oneliner=false);

    public:
        /* constructs the command line parser */
        Cmdline_parser();

        /* Here, the initialization of popt and all the parsing is done.*/
        Configuration* parse(const int argc, const char* argv[]);
};

#endif /* !CMDPARSER_HPP */

